#原画
class OriginalPicturesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  before_filter :authenticate_reader, :only => [:show, :history]
  before_filter :authenticate_user, :only => [:index, :new, :edit, :create, :update, :destroy]
  before_filter :authenticate_artist, :only => [:index, :new, :edit, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]
  
  # GET /original_pictures
  # GET /original_pictures.json
  def index
    @page = OriginalPicture.page params[:page]
    @page_size = OriginalPicture.page_size params[:page_size]
    @original_pictures = OriginalPicture.mylist(@artist.id, @page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @original_pictures.to_json(OriginalPicture.list_json_opt) }
    end
  end

  # GET /original_pictures/1
  # GET /original_pictures/1.json
  def show
    @original_picture = OriginalPicture.show(params[:id], [@artist, @admin])
    
    respond_to do |format|
      opt = {:type => @original_picture.mime_type, :disposition=>"inline"}
      format.png { send_data(@original_picture.restore, opt ) }
      format.gif { send_data(@original_picture.restore, opt ) }
      format.jpeg { send_data(@original_picture.restore, opt ) }
      format.html {
        #ライセンス付与のために原画ライセンスグループ作成準備
        @original_picture_license_group  = OriginalPictureLicenseGroup.new :original_picture_id => @original_picture.id
        render
      }
      format.json { render json: @original_picture.to_json(OriginalPicture.show_json_opt)}
    end
  end

  def history
    @original_picture = OriginalPicture.show(params[:id], [@artist, @admin])
    @history = @original_picture.history
    
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @history.to_json }
    end
  end
  
  def list
    @original_pictures = OriginalPicture.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_pictures }
    end
  end

  def browse
    @original_picture = OriginalPicture.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_picture }
    end
  end

  def new
    @original_picture = OriginalPicture.new
    @original_picture.supply_default

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @original_picture.to_json(OriginalPicture.show_json_opt) }
    end
  end

  def edit
    @original_picture = OriginalPicture.edit(params[:id], @artist)
    respond_to do |format|
      format.html
      format.js
    end
  end

  def create
    @imager = if params[:original_picture]
      PettanImager.load set_image params[:original_picture][:file]
    else
      nil
    end
    @original_picture = OriginalPicture.new
    @original_picture.supply_default
    @original_picture.overwrite @artist

    respond_to do |format|
      if @original_picture.store(@imager)
        flash[:notice] = I18n.t('flash.notice.created', :model => OriginalPicture.model_name.human)
        format.html { redirect_to @original_picture }
        format.json { render json: @original_picture.to_json(OriginalPicture.show_json_opt), status: :created, location: @original_picture }
      else
        format.html { render action: "new" }
        format.json { render json: @original_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @imager = if params[:original_picture]
      PettanImager.load set_image params[:original_picture][:file]
    else
      nil
    end
    @original_picture = OriginalPicture.edit(params[:id], @artist)
    @original_picture.overwrite @artist

    respond_to do |format|
      if @original_picture.store(@imager)
        flash[:notice] = I18n.t('flash.notice.updated', :model => OriginalPicture.model_name.human)
        format.html { redirect_to @original_picture }
        format.json { render json: @original_picture.to_json(OriginalPicture.show_json_opt), status: :created, location: @original_picture }
      else
        format.html { render action: "edit" }
        format.json { render json: @original_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @original_picture = OriginalPicture.edit(params[:id], @artist)
    
    respond_to do |format|
      if @original_picture.destroy_with_resource_picture
        format.html { redirect_to original_pictures_url }
        format.json { head :ok }
      else
        format.html { redirect_to original_picture_path(@original_picture) }
        format.json { render json: @original_picture.errors, status: :unprocessable_entity }
      end
    end
  end
end
