class ProviderStatus < ActiveRecord::Base
  has_one :provider
  
  validates :token, :format => {:with => /^[a-zA-Z0-9]+$/, :allow_blank => true}
  validates :receive_hour1, :numericality => {:allow_blank => true}
  validates :receive_hour2, :numericality => {:allow_blank => true}
#  validates :received_at
#  validates :stopped_at
  
  before_save do |r|
    r.token = r.token.gsub(/\s/, '') if r.token
    r.token = nil if r.token.blank?
  end
  
  def supply_default
  end
  
  def overwrite
  end
  
  def own? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def visible? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def status
    self.token.blank? ? 0 : 1
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(ProviderStatus.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:order => 'providers.name'})
    ProviderStatus.find(:all, opt)
  end
  
  def self.available_list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(ProviderStatus.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['provider_statuses.token is null'], :order => 'providers.name'})
    ProviderStatus.find(:all, opt)
  end
  
  def self.approve_list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(ProviderStatus.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['provider_statuses.token is not null'], :order => 'providers.name'})
    ProviderStatus.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:provider => {}} }
  end
  
  def self.list_json_opt
    {:include => {:provider => {}} }
  end
  
  def self.show pid, ad
    opt = {}
    opt.merge!(ProviderStatus.show_opt)
    res = ProviderStatus.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(ad)
    res
  end
  
  def self.edit pid, ad
    opt = {}
    opt.merge!(ProviderStatus.show_opt)
    res = ProviderStatus.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.own?(ad)
    res
  end
  
  def self.show_opt
    {:include => {:provider => {}} }
  end
  
  def self.show_json_opt
    {:include => {:provider => {}} }
  end
  
  def stop
    self.token = nil
    self.save
  end
  
  def ymd_to_time ymd_str
    return nil if ymd_str.blank?
    date = nil
    begin
      date = Time.parse(ymd_str[0..3] + '/' + ymd_str[4..5] + '/' + ymd_str[6..7])
    rescue
      date = nil
    end
    date
  end
  
  def export_url action, date
    u = self.provider.export_url action
    prm = '?auth_token=' + self.token
    prm = prm + '&date=' + date.strftime("%Y%m%d") unless date.blank?
    u = URI.join(u, prm)
    u.to_s
  end
  
  def export_from_provider url
    res = nil
    begin
      json = RestClient.get url
      res = JSON.parse json
    rescue
    end
    res
  end
  
  def export_by action, ymd
    t = ymd_to_time ymd
    url = export_url action, t
    export_from_provider(url)
  end
  
  def licenses_import date
    licenses = self.export_by('licenses_export', date)
    if  licenses
      ProviderLicense.import self.provider.id, licenses
    else
      LicenseImportResult.new nil
    end
  end
  
  def artists_import date
    artists = self.export_by('artists_export', date)
    if artists
      ProviderArtist.import self.provider.id, artists
    else
      ArtistImportResult.new nil
    end
  end
  
  def original_pictures_import date
    original_pictures = self.export_by('original_pictures_export', date)
    pictures = self.export_by('pictures_export', date)
    if original_pictures and pictures
      ProviderOriginalPicture.import self.provider.id, original_pictures, pictures
    else
      OriginalPictureImportResult.new nil
    end
  end
  
  def import date
    import_result = nil
    ProviderStatus.transaction do
      import_result = self.licenses_import date
      raise ActiveRecord::Rollback unless import_result.success?
      import_result = self.artists_import date
      raise ActiveRecord::Rollback unless import_result.success?
      import_result = self.original_pictures_import date
      raise ActiveRecord::Rollback unless import_result.success?
      self.received_at = Time.now
      self.save
    end
    import_result
  end
  
end
