# -*- encoding: utf-8 -*-
require 'spec_helper'
#色

describe ColorsController do
  before do
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group, :name => 'peta'
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
  end

if MagicNumber['run_mode'] == 1
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @color = FactoryGirl.create :color
      Color.stub(:list).and_return([@color, @color, @color])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Color.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Color.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Color.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '色モデルに一覧を問い合わせている' do
        Color.should_receive(:list).exactly(1)
        get :index
      end
      it '@colorsにリストを取得している' do
        get :index
        assigns(:colors).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '色モデルにjson一覧出力オプションを問い合わせている' do
          Color.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは色っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("name").should be_true
          json.first.has_key?("code").should be_true
          json.first.has_key?("t").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :index
        response.status.should eq 200
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @color = FactoryGirl.create :color
      Color.stub(:show).and_return(@color)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @color.id
        response.should be_success
      end
      it '色モデルに単体取得を問い合わせている' do
        Color.should_receive(:show).exactly(1)
        get :show
      end
      it '@colorにアレを取得している' do
        get :show, :id => @color.id
        assigns(:color).id.should eq(@color.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @color.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @color.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '色モデルにjson単体出力オプションを問い合わせている' do
          Color.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @color.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @color.id, :format => :json
          json = JSON.parse response.body
          json["name"].should_not be_nil
          json["code"].should_not be_nil
          json["t"].should_not be_nil
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :show, :id => @color.id
        response.status.should eq 200
      end
    end
  end
  
else
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @color = FactoryGirl.create :color
      Color.stub(:list).and_return([@color, @color, @color])
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :index
        response.status.should eq 200
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @color = FactoryGirl.create :color
      Color.stub(:show).and_return(@color)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @color.id
        response.should be_success
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @color.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @color.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :show, :id => @color.id
        response.status.should eq 200
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @color.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @color.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
end
end
