# -*- encoding: utf-8 -*-
#セリフ
require 'spec_helper'

describe Speech do
  before do
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    
    @speech_balloon_template = FactoryGirl.create :speech_balloon_template
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.build :speech, :speech_balloon_id => @sb.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @speech.x = 0
        @speech.y = 0
        @speech.width = 1
        @speech.height = 1
        @speech.should be_valid
      end
      it '上限データが通る' do
        @speech.x = 99999
        @speech.y = 99999
        @speech.width = 99999
        @speech.height = 99999
        @speech.should be_valid
      end
    end
    
    context 'speech_balloon_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it 'テストデータの確認' do
        @speech.speech_balloon_id = @sb.id
        @speech.should be_valid
      end
      it '数値でなければ失敗する' do
        @speech.speech_balloon_id = 'a'
        @speech.should_not be_valid
      end
    end
    context 'contentを検証するとき' do
    end
    context 'xを検証するとき' do
      it 'nullなら失敗する' do
        @speech.x = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.x = 'a'
        @speech.should_not be_valid
      end
      it '0なら通る' do
        @speech.x = '0'
        @speech.should be_valid
      end
      it '負でも通る' do
        @speech.x = -1
        @speech.should be_valid
      end
    end
    context 'yを検証するとき' do
      it 'nullなら失敗する' do
        @speech.y = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.y = 'a'
        @speech.should_not be_valid
      end
      it '0なら通る' do
        @speech.y = '0'
        @speech.should be_valid
      end
      it '負でも通る' do
        @speech.y = -1
        @speech.should be_valid
      end
    end
    context 'widthを検証するとき' do
      it 'nullなら失敗する' do
        @speech.width = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.width = 'a'
        @speech.should_not be_valid
      end
      it '0なら失敗する' do
        @speech.width = '0'
        @speech.should_not be_valid
      end
      it '負でも失敗する' do
        @speech.width = -1
        @speech.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      it 'nullなら失敗する' do
        @speech.height = nil
        @speech.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @speech.height = 'a'
        @speech.should_not be_valid
      end
      it '0なら失敗する' do
        @speech.height = '0'
        @speech.should_not be_valid
      end
      it '負でも失敗する' do
        @speech.height = -1
        @speech.should_not be_valid
      end
    end
    context 'settingsを検証するとき' do
    end
  end
  
  describe '文字コード検証に於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.build :speech, :speech_balloon_id => @sb.id
    end
    
    context 'contentを検証するとき' do
      it 'Shift JISなら失敗する' do
        @speech.content = "\x83G\x83r\x83]\x83D"
        lambda{
          @speech.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
    
    context 'settingsを検証するとき' do
      it 'Shift JISなら失敗する' do
        @speech.settings = "\x83G\x83r\x83]\x83D"
        lambda{
          @speech.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.create :speech, :speech_balloon_id => @sb.id
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        Speech.any_instance.stub(:guest_role_check).and_return(true)
        Speech.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @speech.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        Speech.any_instance.stub(:guest_role_check).and_return(false)
        r = @speech.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        Speech.any_instance.stub(:reader_role_check).and_return(true)
        Speech.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @speech.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        Speech.any_instance.stub(:reader_role_check).and_return(false)
        r = @speech.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        Speech.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自身のコマに所持判定を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @speech.visible?([@author])
      end
      it '自身のコマに閲覧許可を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        Panel.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = @speech.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        Panel.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のコマのセリフなら許可する' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @speech.visible?([@author])
        r.should be_true
      end
      it '他人の非公開コマのセリフなら許可しない' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @speech.visible?([@author])
        r.should be_false
      end
      it '他人のコマのセリフでも公開なら許可する' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        r = @speech.visible?([@author])
        r.should be_true
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.create :speech, :speech_balloon_id => @sb.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Speech.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Speech.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Speech.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Speech.page_size('7').should eq 7
      end
      it 'nilの場合はSpeech.default_page_sizeになる' do
        Speech.page_size().should eq Speech.default_page_size
      end
      it '0以下の場合はSpeech.default_page_sizeになる' do
        Speech.page_size('0').should eq Speech.default_page_size
      end
      it 'Speech.max_page_sizeを超えた場合はSpeech.max_page_sizeになる' do
        Speech.page_size('1000').should eq Speech.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Speech.stub(:list_opt).with(any_args).and_return({:include => {:speech_balloon => {:panel => {}}}})
        Speech.should_receive(:list_opt).with(any_args).exactly(1)
        r = Speech.list
      end
    end
    it 'リストを返す' do
      r = Speech.list
      r.should eq [@speech]
    end
    it '時系列で並んでいる' do
      #公開されたコマのセリフは(他人のセリフであっても)含んでいる
      hc = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      nsb = FactoryGirl.create :speech_balloon, :panel_id => hc.id, :speech_balloon_template_id => @speech_balloon_template.id, :updated_at => Time.now + 100
      npl = FactoryGirl.create :speech, :speech_balloon_id => nsb.id, :updated_at => Time.now + 100
      r = Speech.list
      r.should eq [npl, @speech]
    end
    it '非公開のコマのセリフは自分のセリフであっても含まない' do
      hc = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      nsb = FactoryGirl.create :speech_balloon, :panel_id => hc.id, :speech_balloon_template_id => @speech_balloon_template.id, :updated_at => Time.now + 100
      npl = FactoryGirl.create :speech, :speech_balloon_id => nsb.id, :updated_at => Time.now + 100
      pl = Speech.list
      pl.should eq [@speech]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @speech2 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 100
        @speech3 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 200
        @speech4 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 300
        @speech5 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 400
        Speech.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = Speech.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = Speech.list(1)
        pl.should eq [@speech5, @speech4]
      end
      it 'page=2なら中間2件を返す' do
        pl = Speech.list(2)
        pl.should eq [@speech3, @speech2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = Speech.list(3)
        pl.should eq [@speech]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @speech2 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 100
        @speech3 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 200
        @speech4 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 300
        @speech5 = FactoryGirl.create :speech, :speech_balloon_id => @sb.id, :updated_at => Time.now + 400
        Speech.stub(:default_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = Speech.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Speech.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Speech.list_opt[:include]
      r.should have(1).items
    end
    it 'フキダシを含んでいる' do
      r = Speech.list_opt[:include]
      r.has_key?(:speech_balloon).should be_true
    end
      it 'フキダシはコマを含んでいる' do
        r = Speech.list_opt[:include]
        r[:speech_balloon].has_key?(:panel).should be_true
      end
        it 'コマは作家を含んでいる' do
          r = Speech.list_opt[:include]
          r[:speech_balloon][:panel].has_key?(:author).should be_true
        end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Speech.list_opt[:include]
        r[:speech_balloon].has_key?(:balloons).should be_true
      end
      it 'フキダシはフキダシテンプレートを含んでいる' do
        r = Speech.list_opt[:include]
        r[:speech_balloon].has_key?(:speech_balloon_template).should be_true
      end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.create :speech, :speech_balloon_id => @sb.id
    end
    it 'フキダシを含んでいる' do
      r = Speech.list.to_json Speech.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('speech_balloon').should be_true
    end
      it 'フキダシはコマを含んでいる' do
        r = Speech.list.to_json Speech.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['speech_balloon']
        s.has_key?('panel').should be_true
      end
        it 'コマは作家を含んでいる' do
          r = Speech.list.to_json Speech.list_json_opt
          j = JSON.parse r
          i = j.first
          s = i['speech_balloon']['panel']
          s.has_key?('author').should be_true
        end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Speech.list.to_json Speech.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['speech_balloon']
        s.has_key?('balloons').should be_true
      end
      it 'フキダシはフキダシテンプレートを含んでいる' do
        r = Speech.list.to_json Speech.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['speech_balloon']
        s.has_key?('speech_balloon_template').should be_true
      end
  end
  
  describe '単体取得に於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.create :speech, :speech_balloon_id => @sb.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Speech.stub(:show_opt).with(any_args).and_return({:include => {:speech_balloon => {:panel => {}}}})
        Speech.should_receive(:show_opt).with(any_args).exactly(1)
        r = Speech.show @speech.id, @author
      end
      it '閲覧許可を問い合わせている' do
        Speech.any_instance.stub(:visible?).with(any_args).and_return(true)
        Speech.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Speech.show @speech.id, @author
      end
    end
    it '指定のセリフを返す' do
      Speech.any_instance.stub(:visible?).and_return(true)
      pl = Speech.show @speech.id, @author
      pl.should eq @speech
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Speech.any_instance.stub(:visible?).and_return(false)
        lambda{
          Speech.show @speech.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないセリフを開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Speech.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Speech.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Speech.show_opt[:include]
      r.should have(1).items
    end
    it 'フキダシを含んでいる' do
      r = Speech.show_opt[:include]
      r.has_key?(:speech_balloon).should be_true
    end
      it 'フキダシはコマを含んでいる' do
        r = Speech.show_opt[:include]
        r[:speech_balloon].has_key?(:panel).should be_true
      end
        it 'コマは作家を含んでいる' do
          r = Speech.show_opt[:include]
          r[:speech_balloon][:panel].has_key?(:author).should be_true
        end
      it 'フキダシはフキダシ枠を含んでいる' do
        r = Speech.show_opt[:include]
        r[:speech_balloon].has_key?(:balloons).should be_true
      end
      it 'フキダシはフキダシテンプレートを含んでいる' do
        r = Speech.show_opt[:include]
        r[:speech_balloon].has_key?(:speech_balloon_template).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @sb = FactoryGirl.create :speech_balloon, :panel_id => @panel.id, :speech_balloon_template_id => @speech_balloon_template.id
      @balloon = FactoryGirl.create :balloon, :speech_balloon_id => @sb.id, :system_picture_id => @sp.id
      @speech = FactoryGirl.create :speech, :speech_balloon_id => @sb.id
    end
    it 'フキダシを含んでいる' do
      r = Speech.show(@speech.id, @author).to_json Speech.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('speech_balloon').should be_true
    end
      it 'コマを含んでいる' do
        r = Speech.show(@speech.id, @author).to_json Speech.show_json_opt
        j = JSON.parse r
        i = j['speech_balloon']
        i.has_key?('panel').should be_true
      end
        it 'コマは作家を含んでいる' do
          r = Speech.show(@speech.id, @author).to_json Speech.show_json_opt
          j = JSON.parse r
          i = j['speech_balloon']
          s = i['panel']
          s.has_key?('author').should be_true
        end
      it 'フキダシ枠を含んでいる' do
        r = Speech.show(@speech.id, @author).to_json Speech.show_json_opt
        j = JSON.parse r
        i = j['speech_balloon']
        i.has_key?('balloons').should be_true
      end
      it 'フキダシテンプレートを含んでいる' do
        r = Speech.show(@speech.id, @author).to_json Speech.show_json_opt
        j = JSON.parse r
        i = j['speech_balloon']
        i.has_key?('speech_balloon_template').should be_true
      end
  end
end
